@@ -8,11 +8,6 @@
 
 #include "python-zstandard.h"
 
-/* TODO pool.h is a private header and we shouldn't rely on it. */
-#ifndef ZSTD_SINGLE_FILE
-#include "pool.h"
-#endif
-
 extern PyObject *ZstdError;
 
 /**
@@ -34,7 +29,8 @@ int ensure_dctx(ZstdDecompressor *decompressor, int loadDict) {
         }
     }
 
-    zresult = ZSTD_DCtx_setFormat(decompressor->dctx, decompressor->format);
+    zresult = ZSTD_DCtx_setParameter(decompressor->dctx, ZSTD_d_format,
+                                     decompressor->format);
     if (ZSTD_isError(zresult)) {
         PyErr_Format(ZstdError, "unable to set decoding format: %s",
                      ZSTD_getErrorName(zresult));
@@ -63,19 +59,30 @@ static int Decompressor_init(ZstdDecompressor *self, PyObject *args,
                              PyObject *kwargs) {
     static char *kwlist[] = {"dict_data", "max_window_size", "format", NULL};
 
-    ZstdCompressionDict *dict = NULL;
+    PyObject *dict = NULL;
     Py_ssize_t maxWindowSize = 0;
     ZSTD_format_e format = ZSTD_f_zstd1;
 
     self->dctx = NULL;
     self->dict = NULL;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O!nI:ZstdDecompressor",
-                                     kwlist, &ZstdCompressionDictType, &dict,
-                                     &maxWindowSize, &format)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|OnI:ZstdDecompressor",
+                                     kwlist, &dict, &maxWindowSize, &format)) {
         return -1;
     }
 
+    if (dict) {
+        if (dict == Py_None) {
+            dict = NULL;
+        }
+        else if (!PyObject_IsInstance(dict,
+                                      (PyObject *)&ZstdCompressionDictType)) {
+            PyErr_Format(PyExc_TypeError,
+                         "dict_data must be zstd.ZstdCompressionDict");
+            return -1;
+        }
+    }
+
     self->dctx = ZSTD_createDCtx();
     if (!self->dctx) {
         PyErr_NoMemory();
@@ -86,7 +93,7 @@ static int Decompressor_init(ZstdDecompressor *self, PyObject *args,
     self->format = format;
 
     if (dict) {
-        self->dict = dict;
+        self->dict = (ZstdCompressionDict *)dict;
         Py_INCREF(dict);
     }
 
@@ -903,6 +910,7 @@ typedef struct {
     size_t zresult;
 } DecompressorWorkerState;
 
+#ifdef HAVE_ZSTD_POOL_APIS
 static void decompress_worker(DecompressorWorkerState *state) {
     size_t allocationSize;
     DecompressorDestBuffer *destBuffer;
@@ -1144,7 +1152,9 @@ static void decompress_worker(DecompressorWorkerState *state) {
         destBuffer->destSize = destOffset;
     }
 }
+#endif
 
+#ifdef HAVE_ZSTD_POOL_APIS
 ZstdBufferWithSegmentsCollection *
 decompress_from_framesources(ZstdDecompressor *decompressor,
                              FrameSources *frames, Py_ssize_t threadCount) {
@@ -1418,7 +1428,9 @@ finally:
 
     return result;
 }
+#endif
 
+#ifdef HAVE_ZSTD_POOL_APIS
 static ZstdBufferWithSegmentsCollection *
 Decompressor_multi_decompress_to_buffer(ZstdDecompressor *self, PyObject *args,
                                         PyObject *kwargs) {
@@ -1672,6 +1684,7 @@ finally:
 
     return result;
 }
+#endif
 
 static PyMethodDef Decompressor_methods[] = {
     {"copy_stream", (PyCFunction)Decompressor_copy_stream,
@@ -1689,9 +1702,11 @@ static PyMethodDef Decompressor_methods[] = {
     {"decompress_content_dict_chain",
      (PyCFunction)Decompressor_decompress_content_dict_chain,
      METH_VARARGS | METH_KEYWORDS, NULL},
+#ifdef HAVE_ZSTD_POOL_APIS
     {"multi_decompress_to_buffer",
      (PyCFunction)Decompressor_multi_decompress_to_buffer,
      METH_VARARGS | METH_KEYWORDS, NULL},
+#endif
     {"memory_size", (PyCFunction)Decompressor_memory_size, METH_NOARGS, NULL},
     {NULL, NULL}};
 
